var us_chat = new function() {
    this.ws_id = 'us_chat';
    this.app = null;
    this._sanitize_html = require('../../../com_untermstrich_nodejs/node_modules/sanitize-html');
};

/**
 * Init and bind event listener
 * 
 * @param object app        The app
 * @returns void
 */
us_chat.init = function (app) {
    us_chat.app = app;
};

/**
 * Init the client and bind event listener
 * 
 * @param object client     The client object
 * @returns void
 */
us_chat.init_client = function (client) {
    client.on('us_chat_message', function(data) {
        data.created_date = new Date();
        data.message = us_chat.sanitize_html(data.message, true);
        var ret = {
            name:   'us_chat_message',
            data:   data
        };
        
        us_chat.save_message(client.us_data.staff_id, ret.data.message, ret.data.created_date, '');     
        us_chat.app.io.sockets.emit('us_socket', ret);
        us_chat.app.io.sockets.emit('us_socket', {
            name:   'messaging_append_message',
            data:   {
                message: 'CHAT '+ ret.data.staff_name + ': ' + us_chat.sanitize_html(ret.data.message, false),
                ignore_for_staff_id: client.us_data.staff_id
            }
        });        
    });
};

/**
 * Sanitize html
 * 
 * @param string    data       The html data string
 * @returns string
 */
us_chat.sanitize_html = function(data, allow_tags) {
    var allowedTags = [];
    if (allow_tags) {
        allowedTags = [ 'b', 'i', 'em', 'strong' ];
    }
    return us_chat._sanitize_html(data, {
        allowedTags: allowedTags,
        allowedAttributes: {
        }
    });        
};

/**
 * Save a message to database via REST
 * 
 * @param integer   staff_id        The staff ID
 * @param string    msg             The message
 * @param object    created_date    The date
 * @param string    room            Optional a room ID
 * @returns void
 */
us_chat.save_message = function(staff_id, msg, created_date, room) {
    if (typeof(room) === 'undefined') {
        room = null;
    }
    
    //REST call
    us_chat.app.us_rest_call.post(  us_chat.app.conf.vx_base_url + 'rest/node_chat/message/'+us_chat.app.conf.vx_rest_access_token, 
                                    {
                                        "staff_id" :        staff_id,
                                        "message" :         msg,
                                        "created_date" :    created_date,
                                        "room" :            room,
                                    }, 
                                    function(response) {
                                        us_chat.app.logger.info(response);
                                    },
                                    function(err) {
                                        us_chat.app.logger.error(err);
                                    }                                    
    );
};

//Export
module.exports = us_chat;