//"use strict";

/*******************************************************************************/
/**
 * @namespace Global functions of the portlet com_untermstrich_nodejs_chat
 * @version $Id$
 */
var com_untermstrich_nodejs_chat = new function() {
    this.CHAT_ID_GLOBAL = 'GLOBAL';
};

/**
 * Init the butler
 * 
 * @param string portlet_id     The unique portlet ID
 * @returns void
 */
com_untermstrich_nodejs_chat.init = function(portlet_id) {
    if($('#Com_untermstrich_nodejs_chat___'+portlet_id+' .portlet-content').hasClass('initialized')) {
        return;
    }
    $('#Com_untermstrich_nodejs_chat___'+portlet_id+' .nodejs_chat_tabs').tabs({
        tabTemplate: "<li><a href='#{href}'><span>#{label}</span></a><span class='ui-icon ui-icon-close smallbtns ui-icon-closethickgr' style='cursor:pointer'>Remove Tab</span></li>",
        select: function(event, ui) {
           //return main.portlets._on_switch_tab(ui);
        },
        add: function( event, ui ) {
            //var tab_id = main.portlets.get_id_of_tab(ui)
            //$( ui.panel ).append( '<div id="portlet_tabs_'+tab_id+'"></div>' );    
        }
    });
    
    //Modify styling
    $('.nodejs_chat_tabs .ui-tabs-nav li').css('min-height', '23px');
    $('.nodejs_chat_tabs .ui-tabs-nav li a').css('display', 'inline-block');
    $('.nodejs_chat_tabs .ui-tabs-nav li:last a').css('display', 'inline');
    
    $('.chatform').unbind('submit').bind('submit', function() {
        com_untermstrich_nodejs_chat.post_message(this);
    });
    
    com_untermstrich_nodejs_chat._init_node(portlet_id);
    
    $('#Com_untermstrich_nodejs_chat___'+portlet_id+' .portlet-content').addClass('initialized');
};

/**
 * Do node.js initialisation
 * 
 * @param string portlet_id     The unique portlet ID
 * @returns void
 */
com_untermstrich_nodejs_chat._init_node = function(portlet_id) {
    //Add unique hash to differ butlers with same portlet_id (on change portlet tabs)
    var uid = 'a'+Math.random().toString(36).substr(2, 9);
    $('#Com_untermstrich_nodejs_chat___'+portlet_id).addClass(uid);
    
    var stat = usSocket.register_event ('us_chat_message', function(data) {
        return com_untermstrich_nodejs_chat._render_message(data, portlet_id, uid); 
    });
    
    if (!stat) {
        $('.chatform').html('NODE NOT RUNNING');
    } else {
        //Load saved data from db
        com_untermstrich_nodejs_chat._load_messages(portlet_id, uid);
    }
};

/**
 * Load saved messages from DB
 * 
 * @param string    portlet_id      Unique portlet_id
 * @param string    uid             Additional unique ID
 * @param string    room            Optional room ID
 * @returns void
 */
com_untermstrich_nodejs_chat._load_messages = function(portlet_id, uid, room) {
    room = (typeof(room) === 'undefined') ? '' : room;
    openx('portlet', 'com_untermstrich_nodejs_chat', 'load_messages', portlet_id, uid, room);
};

/**
 * Post a message
 * 
 * @param object elem   The input element
 * @returns void
 */
com_untermstrich_nodejs_chat.post_message = function(elem) {
    var msg_field = $(elem).find('.message');
    var msg = msg_field.val();
    
    if (msg !== '') {
        com_untermstrich_nodejs_chat.post_message_do(msg);
    }
    msg_field.val('').focus();
}

/**
 * Do message post
 * 
 * @param string msg    The message
 * @returns void
 */
com_untermstrich_nodejs_chat.post_message_do = function(msg) {
    var data = {
        chat_id: com_untermstrich_nodejs_chat.CHAT_ID_GLOBAL,
        message: msg,
        staff_id: ussettings.current_user,
        staff_name: ussettings.current_user_name,
    }
    usSocket.emit('us_chat_message', data);    
};

/**
 * Render messages from database
 * 
 * @param string    portlet_id      Unique portlet_id
 * @param string    uid             Additional unique ID - needed for portlet tab switching
 * @param object    messages        The messages
 * @param string    room            The room ID
 * @returns void
 */
com_untermstrich_nodejs_chat.render_messages = function(portlet_id, uid, messages, room) {
    var datastring = '';
    
    if (messages.length > 0) {
        var last_date = $.datei18nstring(Date.fromDateString(messages[0].created_date));

        $.each(messages, function(i, val) {
            //add colored date li
            val.created_date = Date.fromDateString(val.created_date);
            if(last_date !== $.datei18nstring(val.created_date) || i === 0) {
                datastring+= '<li class="infobluerow">'+last_date+'</li>';
                last_date = $.datei18nstring(val.created_date);
            }

            //render a message
            datastring+= com_untermstrich_nodejs_chat._render_message_do(val);
        });
    }
    
    var my_portlet = $('#Com_untermstrich_nodejs_chat___'+portlet_id);
    
    if (room == '') {
        var my_loading_span = $(my_portlet).find(' .nodejs_chat_tab_global > span');    
        var my_ul = $(my_portlet).find(' .nodejs_chat_tab_global > form > ul');        
        $(my_loading_span).hide();
        $(my_ul).append(datastring).show();
    }
};

/**
 * Event Callback to render a messages from node.js server
 * 
 * Callbacks have to return true or false to unregister events
 * 
 * @param object    data            Dataobject
 * @param string    portlet_id      Unique portlet_id
 * @param string    uid             Additional unique ID - needed for portlet tab switching
 * @returns true or false
 */
com_untermstrich_nodejs_chat._render_message = function(data, portlet_id, uid) {
    var my_portlet = $('#Com_untermstrich_nodejs_chat___'+portlet_id);
    var my_ul = $(my_portlet).find(' .nodejs_chat_tab_global > form > ul');
    if ($(my_portlet).length === 0 || !$(my_portlet).is(':visible') || !$(my_portlet).hasClass(uid)) {
        return false;
    }
    $(my_ul).append(com_untermstrich_nodejs_chat._render_message_do(data));
    
    $(my_ul).scrollTop($(my_ul)[0].scrollHeight);
    
    return true;
};

/**
 * Create html string
 * 
 * @param object data
 * @returns string
 */
com_untermstrich_nodejs_chat._render_message_do = function(data) {
    var datetime = new Date(data.created_date);
    var li = '<li>';
    var highlight = (data.staff_id == ussettings.current_user) ? 'strong' : 'em';
    
    li+= '<'+highlight+'>'+datetime.getTimeString()+' ';
    li+= data.staff_name+': </'+highlight+'>';
    li+= data.message;
    li+= '</li>';    
    
    return li;
}


